@@ -1,15 +1,29 @@
+from math import fabs
 from random import choice, gauss, randint, random, shuffle
-import math
-from neat.indexer import Indexer
+import sys
+
+# Instead of adding six as a dependency, this code was copied from the six
+# implementation, six is Copyright (c) 2010-2015 Benjamin Peterson
+if sys.version_info[0] == 3:
+    def itervalues(d, **kw):
+        return iter(d.values(**kw))
+
+    def iteritems(d, **kw):
+        return iter(d.items(**kw))
+else:
+    def itervalues(d, **kw):
+        return iter(d.itervalues(**kw))
+
+    def iteritems(d, **kw):
+        return iter(d.iteritems(**kw))
 
 
 class Genome(object):
     """ A genome for general recurrent neural networks. """
-    indexer = Indexer(1)
 
-    def __init__(self, config, parent1_id, parent2_id):
+    def __init__(self, ID, config, parent1_id, parent2_id):
+        self.ID = ID
         self.config = config
-        self.ID = Genome.indexer.next()
         self.num_inputs = config.input_nodes
         self.num_outputs = config.output_nodes
 
@@ -52,7 +66,7 @@ class Genome(object):
 
         return self
 
-    def crossover(self, other):
+    def crossover(self, other, child_id):
         """ Crosses over parents' genomes and returns a child. """
 
         # Parents must belong to the same species.
@@ -68,7 +82,7 @@ class Genome(object):
             parent2 = self
 
         # creates a new child
-        child = self.__class__(self.config, self.ID, other.ID)
+        child = self.__class__(child_id, self.config, self.ID, other.ID)
 
         child.inherit_genes(parent1, parent2)
 
@@ -166,7 +180,7 @@ class Genome(object):
 
         keys_to_delete = set()
         for key, value in self.conn_genes.items():
-            if value.in_node_id == node_id or value.out_node_id == node_id:
+            if node_id in (value.in_node_id, value.out_node_id):
                 keys_to_delete.add(key)
 
         # Do not allow deletion of all connection genes.
@@ -195,61 +209,68 @@ class Genome(object):
     def distance(self, other):
         """ Returns the distance between this genome and the other. """
         if len(self.conn_genes) > len(other.conn_genes):
-            genome1 = self
-            genome2 = other
+            node_genes1 = self.node_genes
+            conn_genes1 = self.conn_genes
+            node_genes2 = other.node_genes
+            conn_genes2 = other.conn_genes
+
         else:
-            genome1 = other
-            genome2 = self
+            node_genes1 = other.node_genes
+            conn_genes1 = other.conn_genes
+            node_genes2 = self.node_genes
+            conn_genes2 = self.conn_genes
 
         # Compute node gene differences.
-        excess1 = sum(1 for k1 in genome1.node_genes if k1 not in genome2.node_genes)
-        excess2 = sum(1 for k2 in genome2.node_genes if k2 not in genome1.node_genes)
-        common_nodes = [k1 for k1 in genome1.node_genes if k1 in genome2.node_genes]
+        excess1 = 0
+        excess2 = sum(1 for k2 in node_genes2 if k2 not in node_genes1)
         bias_diff = 0.0
         response_diff = 0.0
         activation_diff = 0
-        for n in common_nodes:
-            g1 = genome1.node_genes[n]
-            g2 = genome2.node_genes[n]
-            bias_diff += math.fabs(g1.bias - g2.bias)
-            response_diff += math.fabs(g1.response - g2.response)
-            if g1.activation_type != g2.activation_type:
-                activation_diff += 1
+        num_common = 0
+        for k1, g1 in iteritems(node_genes1):
+            if k1 in node_genes2:
+                num_common += 1
+                g2 = node_genes2[k1]
+                bias_diff += fabs(g1.bias - g2.bias)
+                response_diff += fabs(g1.response - g2.response)
+                if g1.activation_type != g2.activation_type:
+                    activation_diff += 1
+            else:
+                excess1 += 1
 
-        most_nodes = max(len(genome1.node_genes), len(genome2.node_genes))
+        most_nodes = max(len(node_genes1), len(node_genes2))
         distance = (self.config.excess_coefficient * float(excess1 + excess2) / most_nodes
                     + self.config.excess_coefficient * float(activation_diff) / most_nodes
-                    + self.config.weight_coefficient * (bias_diff + response_diff) / len(common_nodes))
+                    + self.config.weight_coefficient * (bias_diff + response_diff) / num_common)
 
         # Compute connection gene differences.
-        if genome1.conn_genes:
-            N = len(genome1.conn_genes)
+        if conn_genes1:
+            N = len(conn_genes1)
             weight_diff = 0
             matching = 0
             disjoint = 0
             excess = 0
 
             max_cg_genome2 = None
-            if genome2.conn_genes:
-                max_cg_genome2 = max(genome2.conn_genes.values())
+            if conn_genes2:
+                max_cg_genome2 = max(itervalues(conn_genes2))
 
-            for cg1 in genome1.conn_genes.values():
-                try:
-                    cg2 = genome2.conn_genes[cg1.key]
-                except KeyError:
-                    if max_cg_genome2 is not None and cg1 > max_cg_genome2:
-                        excess += 1
-                    else:
-                        disjoint += 1
-                else:
+            for k1, cg1 in iteritems(conn_genes1):
+                if k1 in conn_genes2:
                     # Homologous genes
-                    weight_diff += math.fabs(cg1.weight - cg2.weight)
+                    cg2 = conn_genes2[k1]
+                    weight_diff += fabs(cg1.weight - cg2.weight)
                     matching += 1
 
                     if cg1.enabled != cg2.enabled:
                         weight_diff += 1.0
+                else:
+                    if max_cg_genome2 is not None and cg1 > max_cg_genome2:
+                        excess += 1
+                    else:
+                        disjoint += 1
 
-            disjoint += len(genome2.conn_genes) - matching
+            disjoint += len(conn_genes2) - matching
 
             distance += self.config.excess_coefficient * float(excess) / N
             distance += self.config.disjoint_coefficient * float(disjoint) / N
@@ -290,10 +311,11 @@ class Genome(object):
             self.node_genes[node_gene.ID] = node_gene
             node_id += 1
 
+    # TODO: Can this be changed to not need a configuration object?
     @classmethod
-    def create_unconnected(cls, config):
+    def create_unconnected(cls, ID, config):
         '''Create a genome for a network with no hidden nodes and no connections.'''
-        c = cls(config, None, None)
+        c = cls(ID, config, None, None)
         node_id = 0
         # Create input node genes.
         for i in range(config.input_nodes):
@@ -326,7 +348,6 @@ class Genome(object):
             cg = self.config.conn_gene_type(ig.ID, og.ID, weight, True)
             self.conn_genes[cg.key] = cg
 
-
     def compute_full_connections(self):
         """ Create a fully-connected genome. """
         in_genes = [g for g in self.node_genes.values() if g.type == 'INPUT']
@@ -369,8 +390,8 @@ class FFGenome(Genome):
         topologies are a particular case of Recurrent NNs.
     """
 
-    def __init__(self, config, parent1_id, parent2_id):
-        super(FFGenome, self).__init__(config, parent1_id, parent2_id)
+    def __init__(self, ID, config, parent1_id, parent2_id):
+        super(FFGenome, self).__init__(ID, config, parent1_id, parent2_id)
         self.node_order = []  # hidden node order
 
     def inherit_genes(self, parent1, parent2):
