@@ -1,16 +1,4 @@
-import math
-
-
-def exp_sigmoid(bias, response, x):
-    z = bias + x * response
-    z = max(-60.0, min(60.0, z))
-    return 1.0 / (1.0 + math.exp(-z))
-
-
-def tanh_sigmoid(bias, response, x):
-    z = bias + x * response
-    z = max(-60.0, min(60.0, z))
-    return math.tanh(z)
+from neat import activation_functions
 
 
 def find_feed_forward_layers(inputs, connections):
@@ -53,6 +41,9 @@ class FeedForwardNetwork(object):
         self.values = [0.0] * (1 + max_node)
 
     def serial_activate(self, inputs):
+        if len(self.input_nodes) != len(inputs):
+            raise Exception("Expected {0} inputs, got {1}".format(len(self.input_nodes), len(inputs)))
+
         for i, v in zip(self.input_nodes, inputs):
             self.values[i] = v
 
@@ -60,7 +51,7 @@ class FeedForwardNetwork(object):
             s = 0.0
             for i, w in links:
                 s += self.values[i] * w
-            self.values[node] = func(bias, response, s)
+            self.values[node] = func(bias + response * s)
 
         return [self.values[i] for i in self.output_nodes]
 
@@ -79,6 +70,7 @@ def create_feed_forward_phenotype(genome):
     for layer in layers:
         for node in layer:
             inputs = []
+            # TODO: This could be more efficient.
             for cg in genome.conn_genes.values():
                 if cg.out_node_id == node and cg.enabled:
                     inputs.append((cg.in_node_id, cg.weight))
@@ -86,44 +78,68 @@ def create_feed_forward_phenotype(genome):
 
             used_nodes.add(node)
             ng = genome.node_genes[node]
-            if ng.activation_type == "tanh":
-                activation_function = tanh_sigmoid
-            else:
-                activation_function = exp_sigmoid
-
+            activation_function = activation_functions.get(ng.activation_type)
             node_evals.append((node, activation_function, ng.bias, ng.response, inputs))
 
     return FeedForwardNetwork(max(used_nodes), input_nodes, output_nodes, node_evals)
 
 
-def create_feed_forward_function(genome):
-    """ Receives a genome and returns a function implementing its neural network. """
+class RecurrentNetwork(object):
+    def __init__(self, max_node, inputs, outputs, node_evals):
+        self.max_node = max_node
+        self.node_evals = node_evals
+        self.input_nodes = inputs
+        self.output_nodes = outputs
+        self.reset()
 
-    f = ['def f(values):']
+    def reset(self):
+        self.values = [[0.0] * (1 + self.max_node),
+                       [0.0] * (1 + self.max_node)]
+        self.active = 0
+
+    def activate(self, inputs):
+        ivalues = self.values[self.active]
+        ovalues = self.values[1 - self.active]
+        self.active = 1 - self.active
+
+        for i, v in zip(self.input_nodes, inputs):
+            ivalues[i] = v
+            ovalues[i] = v
+
+        for node, func, bias, response, links in self.node_evals:
+            s = 0.0
+            for i, w in links:
+                s += ivalues[i] * w
+            ovalues[node] = func(bias + response * s)
+
+        return [ovalues[i] for i in self.output_nodes]
+
+
+def create_recurrent_phenotype(genome):
+    """ Receives a genome and returns its phenotype (a recurrent neural network). """
 
     # Gather inputs and expressed connections.
     input_nodes = [ng.ID for ng in genome.node_genes.values() if ng.type == 'INPUT']
-    connections = [(cg.in_node_id, cg.out_node_id) for cg in genome.conn_genes.values() if cg.enabled]
+    output_nodes = [ng.ID for ng in genome.node_genes.values() if ng.type == 'OUTPUT']
 
-    layers = find_feed_forward_layers(input_nodes, connections)
-    for i, layer in enumerate(layers):
-        f.append('    # evaluate layer %d' % i)
-        for node in layer:
-            ev = []
-            for cg in genome.conn_genes.values():
-                if cg.out_node_id == node and cg.enabled:
-                    ev.append('(%f * values[%d])' % (cg.weight, cg.in_node_id))
+    node_inputs = {}
+    used_nodes = set(input_nodes + output_nodes)
+    for cg in genome.conn_genes.values():
+        if not cg.enabled:
+            continue
 
-            ev = ' + '.join(ev)
-            f.append('    z = ' + ev)
+        used_nodes.add(cg.out_node_id)
+        used_nodes.add(cg.in_node_id)
 
-            ng = genome.node_genes[node]
-            if ng.activation_type == 'tanh':
-                activation_function = 'tanh_sigmoid'
-            else:
-                activation_function = 'exp_sigmoid'
+        if cg.out_node_id not in node_inputs:
+            node_inputs[cg.out_node_id] = [(cg.in_node_id, cg.weight)]
+        else:
+            node_inputs[cg.out_node_id].append((cg.in_node_id, cg.weight))
 
-            f.append('    values[%d] = %s(%f, %f, z)' % (node, activation_function, ng.bias, ng.response))
-            f.append('')
+    node_evals = []
+    for onode, inputs in node_inputs.items():
+        ng = genome.node_genes[onode]
+        activation_function = activation_functions.get(ng.activation_type)
+        node_evals.append((onode, activation_function, ng.bias, ng.response, inputs))
 
-    return '\n'.join(f)
+    return RecurrentNetwork(max(used_nodes), input_nodes, output_nodes, node_evals)
